from __future__ import annotations

from typing import Any


class TotalDaysMismatch(Exception):
    pass


def validate_total_days(form: Any, rx_days: int | None = None) -> None:
    rx_days = rx_days or form.cleaned_data.get("rx_days") or 0
    clinic_days = form.cleaned_data.get("clinic_days") or 0
    club_days = form.cleaned_data.get("club_days") or 0
    purchased_days = form.cleaned_data.get("purchased_days") or 0

    total_days = clinic_days + club_days + purchased_days
    if total_days != rx_days:
        raise TotalDaysMismatch(
            f"Patient to return for a drug refill in {rx_days} days. "
            f"Check that the total days supplied "
            f"({clinic_days} + {club_days} + {purchased_days}) matches this."
        )



from django.forms import Form
from django.test import TestCase

from edc_rx.utils import TotalDaysMismatch, validate_total_days


class TestRxUtils(TestCase):
    def test_validate_total_days_mismatch_raises(self):
        form = Form()
        for invalid_combo in [
            (0, 0, 0),
            (None, None, None),
            (1, 1, 1),
            (20, 10, 1),
            (1, 20, 0),
            (1, 20, None),
            (30, 30, 30),
            (31, 0, 0),
            (31, None, None),
        ]:
            clinic_days = invalid_combo[0]
            club_days = invalid_combo[1]
            purchased_days = invalid_combo[2]
            with self.subTest(
                clinic_days=clinic_days,
                club_days=club_days,
                purchased_days=purchased_days,
            ):
                form.cleaned_data = {
                    "rx_days": 30,
                    "clinic_days": clinic_days,
                    "club_days": club_days,
                    "purchased_days": purchased_days,
                }
                with self.assertRaises(TotalDaysMismatch):
                    validate_total_days(form=form)

    def test_validate_total_days_match_ok(self):
        form = Form()
        for valid_combo in [
            (30, 0, 0),
            (0, 30, 0),
            (0, 0, 30),
            (30, 0, None),
            (30, None, None),
            (20, 10, 0),
            (20, 10, None),
            (0, 10, 20),
            (None, 10, 20),
            (10, 10, 10),
            (5, 20, 5),
        ]:
            clinic_days = valid_combo[0]
            club_days = valid_combo[1]
            purchased_days = valid_combo[2]
            with self.subTest(
                clinic_days=clinic_days,
                club_days=club_days,
                purchased_days=purchased_days,
            ):
                form.cleaned_data = {
                    "rx_days": 30,
                    "clinic_days": clinic_days,
                    "club_days": club_days,
                    "purchased_days": purchased_days,
                }
                try:
                    validate_total_days(form=form)
                except TotalDaysMismatch as e:
                    self.fail(f"TotalDaysMismatch unexpectedly raised. Got {e}")

    def test_validate_total_days_0_with_mismatch_raises(self):
        form = Form()
        for invalid_combo in [
            (1, 1, 1),
            (20, 10, 1),
            (1, 20, 0),
            (1, 20, None),
            (30, 30, 30),
            (31, 0, 0),
            (31, None, None),
        ]:
            clinic_days = invalid_combo[0]
            club_days = invalid_combo[1]
            purchased_days = invalid_combo[2]
            with self.subTest(
                clinic_days=clinic_days,
                club_days=club_days,
                purchased_days=purchased_days,
            ):
                form.cleaned_data = {
                    "rx_days": 0,
                    "clinic_days": clinic_days,
                    "club_days": club_days,
                    "purchased_days": purchased_days,
                }
                with self.assertRaises(TotalDaysMismatch):
                    validate_total_days(form=form)

    def test_validate_total_days_0_with_match_ok(self):
        form = Form()
        for valid_combo in [
            (0, 0, 0),
            (None, None, None),
            (0, None, 0),
            (None, 0, None),
        ]:
            clinic_days = valid_combo[0]
            club_days = valid_combo[1]
            purchased_days = valid_combo[2]
            with self.subTest(
                clinic_days=clinic_days,
                club_days=club_days,
                purchased_days=purchased_days,
            ):
                form.cleaned_data = {
                    "rx_days": 0,
                    "clinic_days": clinic_days,
                    "club_days": club_days,
                    "purchased_days": purchased_days,
                }
                try:
                    validate_total_days(form=form)
                except TotalDaysMismatch as e:
                    self.fail(f"TotalDaysMismatch unexpectedly raised. Got {e}")



from django.contrib import admin
from django.utils.html import format_html
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple


class DrugRefillAdminMixin:
    form = None

    inlines = []

    fieldsets_move_to_end = [crf_status_fieldset_tuple[0], audit_fieldset_tuple[0]]

    additional_instructions = format_html(
        '<span style="color:orange">Note: Medications CRF must be completed first.</span>'
    )

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Refill Information",
            {
                "fields": (
                    "rx",
                    "rx_other",
                    "rx_modified",
                    "modifications",
                    "modifications_other",
                    "modifications_reason",
                    "modifications_reason_other",
                    "rx_days",
                )
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )
    filter_horizontal = ["rx", "modifications", "modifications_reason"]

    radio_fields = {
        "crf_status": admin.VERTICAL,
        "rx_modified": admin.VERTICAL,
    }



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models


class DrugSupplyModelMixin(models.Model):
    clinic_days = models.IntegerField(
        verbose_name="Clinic",
        validators=[MinValueValidator(0), MaxValueValidator(180)],
        help_text="days",
    )

    club_days = models.IntegerField(
        verbose_name="Club",
        validators=[MinValueValidator(0), MaxValueValidator(180)],
        help_text="days",
    )

    purchased_days = models.IntegerField(
        verbose_name="To purchase",
        validators=[MinValueValidator(0), MaxValueValidator(180)],
        help_text=(
            "This can be purchased by patient, through a medicines club "
            "that the patient belong to, through insurance or someone else has paid. "
        ),
    )

    class Meta:
        abstract = True
